# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `API.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_cmd_API_ice
import omero_ServerErrors_ice
import omero_ModelF_ice
import omero_ServicesF_ice
import omero_System_ice
import omero_Collections_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module Glacier2
_M_Glacier2 = Ice.openModule('Glacier2')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Included module omero.cmd
_M_omero.cmd = Ice.openModule('omero.cmd')

# Included module omero.grid
_M_omero.grid = Ice.openModule('omero.grid')

# Start of module omero
__name__ = 'omero'

# Start of module omero.api
__name__ = 'omero.api'
_M_omero.api.__doc__ = """
The omero::api module defines all the central verbs for working with OMERO.blitz.
 Arguments and return values consist of those
types defined in the other ice files available here. With no
further custom code, it is possible to interoperate with
OMERO.blitz simply via the definitions here. Start with the
ServiceFactory definition at the end of this file.
 Note: Using these types is significantly easier in combination with
the JavaDocs of the OMERO.server, specifically the ome.api
package. Where not further noted below, the follow mappings between
ome.api argument types and omero::api argument types hold: 
+-----------------------+------------------------+
|        ome.api        |      omero::api        |
+-----------------------+------------------------+
|java.lang.Class        |string                  |
+-----------------------+------------------------+
|java.util.Set          |java.util.List/vector   |
+-----------------------+------------------------+
|IPojo options (Map)    |omero::sys::ParamMap    |
+-----------------------+------------------------+
|If null needed         |omero::RType subclass   |
+-----------------------+------------------------+
|...                    |...                     |
+-----------------------+------------------------+
"""

if 'ClientCallback' not in _M_omero.api.__dict__:
    _M_omero.api.ClientCallback = Ice.createTempClass()
    class ClientCallback(Ice.Object):
        """
        Primary callback interface for interaction between client and
        server session (""ServiceFactory""). Where possible these methods
        will be called one-way to prevent clients from hanging the server.
        """
        def __init__(self):
            if Ice.getType(self) == _M_omero.api.ClientCallback:
                raise RuntimeError('omero.api.ClientCallback is an abstract class')

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::api::ClientCallback')

        def ice_id(self, current=None):
            return '::omero::api::ClientCallback'

        def ice_staticId():
            return '::omero::api::ClientCallback'
        ice_staticId = staticmethod(ice_staticId)

        def requestHeartbeat(self, current=None):
            """
            Heartbeat-request made by the server to guarantee that the client
            is alive. If the client is still active, then some method should
            be made on the server to update the last idle time.
            Arguments:
            current -- The Current object for the invocation.
            """
            pass

        def sessionClosed(self, current=None):
            """
            The session to which this ServiceFactory is connected has been
            closed. Almost no further method calls (if any) are possible.
            Create a new session via omero.client.createSession()
            Arguments:
            current -- The Current object for the invocation.
            """
            pass

        def shutdownIn(self, milliseconds, current=None):
            """
            Message that the server will be shutting down in the
            given number of milliseconds, after which all new and
            running method invocations will receive a CancelledException.
            Arguments:
            milliseconds -- 
            current -- The Current object for the invocation.
            """
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.api._t_ClientCallback)

        __repr__ = __str__

    _M_omero.api.ClientCallbackPrx = Ice.createTempClass()
    class ClientCallbackPrx(Ice.ObjectPrx):

        """
        Heartbeat-request made by the server to guarantee that the client
        is alive. If the client is still active, then some method should
        be made on the server to update the last idle time.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def requestHeartbeat(self, _ctx=None):
            return _M_omero.api.ClientCallback._op_requestHeartbeat.invoke(self, ((), _ctx))

        """
        Heartbeat-request made by the server to guarantee that the client
        is alive. If the client is still active, then some method should
        be made on the server to update the last idle time.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_requestHeartbeat(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ClientCallback._op_requestHeartbeat.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Heartbeat-request made by the server to guarantee that the client
        is alive. If the client is still active, then some method should
        be made on the server to update the last idle time.
        Arguments:
        """
        def end_requestHeartbeat(self, _r):
            return _M_omero.api.ClientCallback._op_requestHeartbeat.end(self, _r)

        """
        The session to which this ServiceFactory is connected has been
        closed. Almost no further method calls (if any) are possible.
        Create a new session via omero.client.createSession()
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def sessionClosed(self, _ctx=None):
            return _M_omero.api.ClientCallback._op_sessionClosed.invoke(self, ((), _ctx))

        """
        The session to which this ServiceFactory is connected has been
        closed. Almost no further method calls (if any) are possible.
        Create a new session via omero.client.createSession()
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_sessionClosed(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ClientCallback._op_sessionClosed.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        The session to which this ServiceFactory is connected has been
        closed. Almost no further method calls (if any) are possible.
        Create a new session via omero.client.createSession()
        Arguments:
        """
        def end_sessionClosed(self, _r):
            return _M_omero.api.ClientCallback._op_sessionClosed.end(self, _r)

        """
        Message that the server will be shutting down in the
        given number of milliseconds, after which all new and
        running method invocations will receive a CancelledException.
        Arguments:
        milliseconds -- 
        _ctx -- The request context for the invocation.
        """
        def shutdownIn(self, milliseconds, _ctx=None):
            return _M_omero.api.ClientCallback._op_shutdownIn.invoke(self, ((milliseconds, ), _ctx))

        """
        Message that the server will be shutting down in the
        given number of milliseconds, after which all new and
        running method invocations will receive a CancelledException.
        Arguments:
        milliseconds -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_shutdownIn(self, milliseconds, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ClientCallback._op_shutdownIn.begin(self, ((milliseconds, ), _response, _ex, _sent, _ctx))

        """
        Message that the server will be shutting down in the
        given number of milliseconds, after which all new and
        running method invocations will receive a CancelledException.
        Arguments:
        milliseconds -- 
        """
        def end_shutdownIn(self, _r):
            return _M_omero.api.ClientCallback._op_shutdownIn.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.api.ClientCallbackPrx.ice_checkedCast(proxy, '::omero::api::ClientCallback', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.api.ClientCallbackPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::api::ClientCallback'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.api._t_ClientCallbackPrx = IcePy.defineProxy('::omero::api::ClientCallback', ClientCallbackPrx)

    _M_omero.api._t_ClientCallback = IcePy.defineClass('::omero::api::ClientCallback', ClientCallback, -1, (), True, False, None, (), ())
    ClientCallback._ice_type = _M_omero.api._t_ClientCallback

    ClientCallback._op_requestHeartbeat = IcePy.Operation('requestHeartbeat', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    ClientCallback._op_sessionClosed = IcePy.Operation('sessionClosed', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    ClientCallback._op_shutdownIn = IcePy.Operation('shutdownIn', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_long, False, 0),), (), None, ())

    _M_omero.api.ClientCallback = ClientCallback
    del ClientCallback

    _M_omero.api.ClientCallbackPrx = ClientCallbackPrx
    del ClientCallbackPrx

if 'ServiceFactory' not in _M_omero.api.__dict__:
    _M_omero.api.ServiceFactory = Ice.createTempClass()
    class ServiceFactory(_M_omero.cmd.Session):
        """
        Starting point for all OMERO.blitz interaction.
         A ServiceFactory once acquired can be used to create any number
        of service proxies to the server. Most services implement
        ServiceInterface or its subinterface
        StatefulServiceInterface.
        """
        def __init__(self):
            if Ice.getType(self) == _M_omero.api.ServiceFactory:
                raise RuntimeError('omero.api.ServiceFactory is an abstract class')

        def ice_ids(self, current=None):
            return ('::Glacier2::Session', '::Ice::Object', '::omero::api::ServiceFactory', '::omero::cmd::Session')

        def ice_id(self, current=None):
            return '::omero::api::ServiceFactory'

        def ice_staticId():
            return '::omero::api::ServiceFactory'
        ice_staticId = staticmethod(ice_staticId)

        def getSecurityContexts(self, current=None):
            """
            Provides a list of all valid security contexts for this session.
            Each of the returned omero.model.IObject instances can
            be passed to {@code setSecurityContext}.
            Arguments:
            current -- The Current object for the invocation.
            """
            pass

        def setSecurityContext(self, obj, current=None):
            """
            Changes the security context for the current session.
             A security context limits the set of objects which will
            be returned by all queries and restricts what updates
            can be made. 
             Current valid values for security context:
            omero.model.ExperimenterGroup - logs into a
            specific group
            omero.model.Share - uses IShare to activate a
            share
             Passing an unloaded version of either object type will change
            the way the current session operates. Note: only objects which
            are returned by the {@code getSecurityContext} method are
            considered valid. Any other instance will cause an exception to
            be thrown. 
            Example usage in Python:
            sf = client.createSession()
            objs = sf.getSecurityContexts()
            old = sf.setSecurityContext(objs\[-1])
            Arguments:
            obj -- 
            current -- The Current object for the invocation.
            """
            pass

        def setSecurityPassword(self, password, current=None):
            """
            Re-validates the password for the current session. This prevents
            See methods that mention "HasPassword".
            Arguments:
            password -- 
            current -- The Current object for the invocation.
            """
            pass

        def getAdminService(self, current=None):
            pass

        def getConfigService(self, current=None):
            pass

        def getContainerService(self, current=None):
            pass

        def getLdapService(self, current=None):
            pass

        def getPixelsService(self, current=None):
            pass

        def getProjectionService(self, current=None):
            pass

        def getQueryService(self, current=None):
            pass

        def getRenderingSettingsService(self, current=None):
            pass

        def getRepositoryInfoService(self, current=None):
            pass

        def getRoiService(self, current=None):
            pass

        def getScriptService(self, current=None):
            pass

        def getSessionService(self, current=None):
            pass

        def getShareService(self, current=None):
            pass

        def getTimelineService(self, current=None):
            pass

        def getTypesService(self, current=None):
            pass

        def getUpdateService(self, current=None):
            pass

        def getMetadataService(self, current=None):
            pass

        def createExporter(self, current=None):
            pass

        def createJobHandle(self, current=None):
            pass

        def createRawFileStore(self, current=None):
            pass

        def createRawPixelsStore(self, current=None):
            pass

        def createRenderingEngine(self, current=None):
            pass

        def createSearchService(self, current=None):
            pass

        def createThumbnailStore(self, current=None):
            pass

        def sharedResources(self, current=None):
            """
            Returns a reference to a back-end manager. The
            omero.grid.SharedResources service provides look ups
            for various facilities offered by OMERO:
            OMERO.scripts
            OMERO.tables
            These facilities may or may not be available on first request.
            Arguments:
            current -- The Current object for the invocation.
            """
            pass

        def getByName(self, name, current=None):
            """
            Allows looking up any stateless service by name.
            See Constants.ice for examples of services.
            If a service has been added by third-parties,
            getByName can be used even though no concrete
            method is available.
            Arguments:
            name -- 
            current -- The Current object for the invocation.
            """
            pass

        def createByName(self, name, current=None):
            """
            Allows looking up any stateful service by name.
            See Constants.ice for examples of services.
            If a service has been added by third-parties,
            createByName can be used even though no concrete
            method is available.
            Arguments:
            name -- 
            current -- The Current object for the invocation.
            """
            pass

        def subscribe(self, topicName, prx, current=None):
            """
            Subscribe to a given topic. The topic must exist and the user must
            have sufficient permissions for that topic. Further the proxy object
            must match the required type for the topic as encoded in the topic
            name.
            Arguments:
            topicName -- 
            prx -- 
            current -- The Current object for the invocation.
            """
            pass

        def setCallback(self, callback, current=None):
            """
            Sets the single callback used by the ServiceFactory
            to communicate with the client application. A default
            callback is set by the omero::client object on
            session creation which should suffice for most usage.
            See the client object's documentation in each language
            mapping for ways to use the callback.
            Arguments:
            callback -- 
            current -- The Current object for the invocation.
            """
            pass

        def closeOnDestroy(self, current=None):
            """
            Marks the session for closure rather than detachment, which will
            be triggered by the destruction of the Glacier2 connection via
            router.destroySession()
            Closing the session rather the detaching is more secure, since all
            resources are removed from the server and can safely be set once
            it is clear that a client is finished with those resources.
            Arguments:
            current -- The Current object for the invocation.
            """
            pass

        def detachOnDestroy(self, current=None):
            """
            Marks the session for detachment rather than closure, which will
            be triggered by the destruction of the Glacier2 connection via
            router.destroySession()
            This is the default and allows a lost session to be reconnected,
            at a slight security cost since the session will persist longer
            and can be used by others if the UUID is intercepted.
            Arguments:
            current -- The Current object for the invocation.
            """
            pass

        def activeServices(self, current=None):
            """
            Returns a list of string ids for currently active services. This will
            _not_ keep services alive, and in fact checks for all expired services
            and removes them.
            Arguments:
            current -- The Current object for the invocation.
            """
            pass

        def keepAllAlive(self, proxies, current=None):
            """
            Requests that the given services be marked as alive. It is
            possible that one of the services has already timed out, in which
            case the returned long value will be non-zero.
            Specifically, the bit representing the 0-based index will be 1:
            {@code
            if (retval & 1&lt;&lt;idx == 1&lt;&lt;idx) { // not alive }
            }
            Except for fatal server or session errors, this method should never
            throw an exception.
            Arguments:
            proxies -- 
            current -- The Current object for the invocation.
            """
            pass

        def keepAlive(self, proxy, current=None):
            """
            Returns true if the given service is alive.
            Except for fatal server or session errors, this method should never
            throw an exception.
            Arguments:
            proxy -- 
            current -- The Current object for the invocation.
            """
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.api._t_ServiceFactory)

        __repr__ = __str__

    _M_omero.api.ServiceFactoryPrx = Ice.createTempClass()
    class ServiceFactoryPrx(_M_omero.cmd.SessionPrx):

        """
        Provides a list of all valid security contexts for this session.
        Each of the returned omero.model.IObject instances can
        be passed to {@code setSecurityContext}.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def getSecurityContexts(self, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getSecurityContexts.invoke(self, ((), _ctx))

        """
        Provides a list of all valid security contexts for this session.
        Each of the returned omero.model.IObject instances can
        be passed to {@code setSecurityContext}.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getSecurityContexts(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getSecurityContexts.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Provides a list of all valid security contexts for this session.
        Each of the returned omero.model.IObject instances can
        be passed to {@code setSecurityContext}.
        Arguments:
        """
        def end_getSecurityContexts(self, _r):
            return _M_omero.api.ServiceFactory._op_getSecurityContexts.end(self, _r)

        """
        Changes the security context for the current session.
         A security context limits the set of objects which will
        be returned by all queries and restricts what updates
        can be made. 
         Current valid values for security context:
        omero.model.ExperimenterGroup - logs into a
        specific group
        omero.model.Share - uses IShare to activate a
        share
         Passing an unloaded version of either object type will change
        the way the current session operates. Note: only objects which
        are returned by the {@code getSecurityContext} method are
        considered valid. Any other instance will cause an exception to
        be thrown. 
        Example usage in Python:
        sf = client.createSession()
        objs = sf.getSecurityContexts()
        old = sf.setSecurityContext(objs\[-1])
        Arguments:
        obj -- 
        _ctx -- The request context for the invocation.
        """
        def setSecurityContext(self, obj, _ctx=None):
            return _M_omero.api.ServiceFactory._op_setSecurityContext.invoke(self, ((obj, ), _ctx))

        """
        Changes the security context for the current session.
         A security context limits the set of objects which will
        be returned by all queries and restricts what updates
        can be made. 
         Current valid values for security context:
        omero.model.ExperimenterGroup - logs into a
        specific group
        omero.model.Share - uses IShare to activate a
        share
         Passing an unloaded version of either object type will change
        the way the current session operates. Note: only objects which
        are returned by the {@code getSecurityContext} method are
        considered valid. Any other instance will cause an exception to
        be thrown. 
        Example usage in Python:
        sf = client.createSession()
        objs = sf.getSecurityContexts()
        old = sf.setSecurityContext(objs\[-1])
        Arguments:
        obj -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_setSecurityContext(self, obj, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_setSecurityContext.begin(self, ((obj, ), _response, _ex, _sent, _ctx))

        """
        Changes the security context for the current session.
         A security context limits the set of objects which will
        be returned by all queries and restricts what updates
        can be made. 
         Current valid values for security context:
        omero.model.ExperimenterGroup - logs into a
        specific group
        omero.model.Share - uses IShare to activate a
        share
         Passing an unloaded version of either object type will change
        the way the current session operates. Note: only objects which
        are returned by the {@code getSecurityContext} method are
        considered valid. Any other instance will cause an exception to
        be thrown. 
        Example usage in Python:
        sf = client.createSession()
        objs = sf.getSecurityContexts()
        old = sf.setSecurityContext(objs\[-1])
        Arguments:
        obj -- 
        """
        def end_setSecurityContext(self, _r):
            return _M_omero.api.ServiceFactory._op_setSecurityContext.end(self, _r)

        """
        Re-validates the password for the current session. This prevents
        See methods that mention "HasPassword".
        Arguments:
        password -- 
        _ctx -- The request context for the invocation.
        """
        def setSecurityPassword(self, password, _ctx=None):
            return _M_omero.api.ServiceFactory._op_setSecurityPassword.invoke(self, ((password, ), _ctx))

        """
        Re-validates the password for the current session. This prevents
        See methods that mention "HasPassword".
        Arguments:
        password -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_setSecurityPassword(self, password, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_setSecurityPassword.begin(self, ((password, ), _response, _ex, _sent, _ctx))

        """
        Re-validates the password for the current session. This prevents
        See methods that mention "HasPassword".
        Arguments:
        password -- 
        """
        def end_setSecurityPassword(self, _r):
            return _M_omero.api.ServiceFactory._op_setSecurityPassword.end(self, _r)

        def getAdminService(self, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getAdminService.invoke(self, ((), _ctx))

        def begin_getAdminService(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getAdminService.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getAdminService(self, _r):
            return _M_omero.api.ServiceFactory._op_getAdminService.end(self, _r)

        def getConfigService(self, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getConfigService.invoke(self, ((), _ctx))

        def begin_getConfigService(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getConfigService.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getConfigService(self, _r):
            return _M_omero.api.ServiceFactory._op_getConfigService.end(self, _r)

        def getContainerService(self, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getContainerService.invoke(self, ((), _ctx))

        def begin_getContainerService(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getContainerService.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getContainerService(self, _r):
            return _M_omero.api.ServiceFactory._op_getContainerService.end(self, _r)

        def getLdapService(self, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getLdapService.invoke(self, ((), _ctx))

        def begin_getLdapService(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getLdapService.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getLdapService(self, _r):
            return _M_omero.api.ServiceFactory._op_getLdapService.end(self, _r)

        def getPixelsService(self, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getPixelsService.invoke(self, ((), _ctx))

        def begin_getPixelsService(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getPixelsService.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getPixelsService(self, _r):
            return _M_omero.api.ServiceFactory._op_getPixelsService.end(self, _r)

        def getProjectionService(self, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getProjectionService.invoke(self, ((), _ctx))

        def begin_getProjectionService(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getProjectionService.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getProjectionService(self, _r):
            return _M_omero.api.ServiceFactory._op_getProjectionService.end(self, _r)

        def getQueryService(self, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getQueryService.invoke(self, ((), _ctx))

        def begin_getQueryService(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getQueryService.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getQueryService(self, _r):
            return _M_omero.api.ServiceFactory._op_getQueryService.end(self, _r)

        def getRenderingSettingsService(self, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getRenderingSettingsService.invoke(self, ((), _ctx))

        def begin_getRenderingSettingsService(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getRenderingSettingsService.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getRenderingSettingsService(self, _r):
            return _M_omero.api.ServiceFactory._op_getRenderingSettingsService.end(self, _r)

        def getRepositoryInfoService(self, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getRepositoryInfoService.invoke(self, ((), _ctx))

        def begin_getRepositoryInfoService(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getRepositoryInfoService.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getRepositoryInfoService(self, _r):
            return _M_omero.api.ServiceFactory._op_getRepositoryInfoService.end(self, _r)

        def getRoiService(self, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getRoiService.invoke(self, ((), _ctx))

        def begin_getRoiService(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getRoiService.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getRoiService(self, _r):
            return _M_omero.api.ServiceFactory._op_getRoiService.end(self, _r)

        def getScriptService(self, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getScriptService.invoke(self, ((), _ctx))

        def begin_getScriptService(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getScriptService.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getScriptService(self, _r):
            return _M_omero.api.ServiceFactory._op_getScriptService.end(self, _r)

        def getSessionService(self, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getSessionService.invoke(self, ((), _ctx))

        def begin_getSessionService(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getSessionService.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getSessionService(self, _r):
            return _M_omero.api.ServiceFactory._op_getSessionService.end(self, _r)

        def getShareService(self, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getShareService.invoke(self, ((), _ctx))

        def begin_getShareService(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getShareService.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getShareService(self, _r):
            return _M_omero.api.ServiceFactory._op_getShareService.end(self, _r)

        def getTimelineService(self, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getTimelineService.invoke(self, ((), _ctx))

        def begin_getTimelineService(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getTimelineService.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getTimelineService(self, _r):
            return _M_omero.api.ServiceFactory._op_getTimelineService.end(self, _r)

        def getTypesService(self, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getTypesService.invoke(self, ((), _ctx))

        def begin_getTypesService(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getTypesService.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getTypesService(self, _r):
            return _M_omero.api.ServiceFactory._op_getTypesService.end(self, _r)

        def getUpdateService(self, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getUpdateService.invoke(self, ((), _ctx))

        def begin_getUpdateService(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getUpdateService.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getUpdateService(self, _r):
            return _M_omero.api.ServiceFactory._op_getUpdateService.end(self, _r)

        def getMetadataService(self, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getMetadataService.invoke(self, ((), _ctx))

        def begin_getMetadataService(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getMetadataService.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getMetadataService(self, _r):
            return _M_omero.api.ServiceFactory._op_getMetadataService.end(self, _r)

        def createExporter(self, _ctx=None):
            return _M_omero.api.ServiceFactory._op_createExporter.invoke(self, ((), _ctx))

        def begin_createExporter(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_createExporter.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_createExporter(self, _r):
            return _M_omero.api.ServiceFactory._op_createExporter.end(self, _r)

        def createJobHandle(self, _ctx=None):
            return _M_omero.api.ServiceFactory._op_createJobHandle.invoke(self, ((), _ctx))

        def begin_createJobHandle(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_createJobHandle.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_createJobHandle(self, _r):
            return _M_omero.api.ServiceFactory._op_createJobHandle.end(self, _r)

        def createRawFileStore(self, _ctx=None):
            return _M_omero.api.ServiceFactory._op_createRawFileStore.invoke(self, ((), _ctx))

        def begin_createRawFileStore(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_createRawFileStore.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_createRawFileStore(self, _r):
            return _M_omero.api.ServiceFactory._op_createRawFileStore.end(self, _r)

        def createRawPixelsStore(self, _ctx=None):
            return _M_omero.api.ServiceFactory._op_createRawPixelsStore.invoke(self, ((), _ctx))

        def begin_createRawPixelsStore(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_createRawPixelsStore.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_createRawPixelsStore(self, _r):
            return _M_omero.api.ServiceFactory._op_createRawPixelsStore.end(self, _r)

        def createRenderingEngine(self, _ctx=None):
            return _M_omero.api.ServiceFactory._op_createRenderingEngine.invoke(self, ((), _ctx))

        def begin_createRenderingEngine(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_createRenderingEngine.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_createRenderingEngine(self, _r):
            return _M_omero.api.ServiceFactory._op_createRenderingEngine.end(self, _r)

        def createSearchService(self, _ctx=None):
            return _M_omero.api.ServiceFactory._op_createSearchService.invoke(self, ((), _ctx))

        def begin_createSearchService(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_createSearchService.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_createSearchService(self, _r):
            return _M_omero.api.ServiceFactory._op_createSearchService.end(self, _r)

        def createThumbnailStore(self, _ctx=None):
            return _M_omero.api.ServiceFactory._op_createThumbnailStore.invoke(self, ((), _ctx))

        def begin_createThumbnailStore(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_createThumbnailStore.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_createThumbnailStore(self, _r):
            return _M_omero.api.ServiceFactory._op_createThumbnailStore.end(self, _r)

        """
        Returns a reference to a back-end manager. The
        omero.grid.SharedResources service provides look ups
        for various facilities offered by OMERO:
        OMERO.scripts
        OMERO.tables
        These facilities may or may not be available on first request.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def sharedResources(self, _ctx=None):
            return _M_omero.api.ServiceFactory._op_sharedResources.invoke(self, ((), _ctx))

        """
        Returns a reference to a back-end manager. The
        omero.grid.SharedResources service provides look ups
        for various facilities offered by OMERO:
        OMERO.scripts
        OMERO.tables
        These facilities may or may not be available on first request.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_sharedResources(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_sharedResources.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Returns a reference to a back-end manager. The
        omero.grid.SharedResources service provides look ups
        for various facilities offered by OMERO:
        OMERO.scripts
        OMERO.tables
        These facilities may or may not be available on first request.
        Arguments:
        """
        def end_sharedResources(self, _r):
            return _M_omero.api.ServiceFactory._op_sharedResources.end(self, _r)

        """
        Allows looking up any stateless service by name.
        See Constants.ice for examples of services.
        If a service has been added by third-parties,
        getByName can be used even though no concrete
        method is available.
        Arguments:
        name -- 
        _ctx -- The request context for the invocation.
        """
        def getByName(self, name, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getByName.invoke(self, ((name, ), _ctx))

        """
        Allows looking up any stateless service by name.
        See Constants.ice for examples of services.
        If a service has been added by third-parties,
        getByName can be used even though no concrete
        method is available.
        Arguments:
        name -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getByName(self, name, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_getByName.begin(self, ((name, ), _response, _ex, _sent, _ctx))

        """
        Allows looking up any stateless service by name.
        See Constants.ice for examples of services.
        If a service has been added by third-parties,
        getByName can be used even though no concrete
        method is available.
        Arguments:
        name -- 
        """
        def end_getByName(self, _r):
            return _M_omero.api.ServiceFactory._op_getByName.end(self, _r)

        """
        Allows looking up any stateful service by name.
        See Constants.ice for examples of services.
        If a service has been added by third-parties,
        createByName can be used even though no concrete
        method is available.
        Arguments:
        name -- 
        _ctx -- The request context for the invocation.
        """
        def createByName(self, name, _ctx=None):
            return _M_omero.api.ServiceFactory._op_createByName.invoke(self, ((name, ), _ctx))

        """
        Allows looking up any stateful service by name.
        See Constants.ice for examples of services.
        If a service has been added by third-parties,
        createByName can be used even though no concrete
        method is available.
        Arguments:
        name -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_createByName(self, name, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_createByName.begin(self, ((name, ), _response, _ex, _sent, _ctx))

        """
        Allows looking up any stateful service by name.
        See Constants.ice for examples of services.
        If a service has been added by third-parties,
        createByName can be used even though no concrete
        method is available.
        Arguments:
        name -- 
        """
        def end_createByName(self, _r):
            return _M_omero.api.ServiceFactory._op_createByName.end(self, _r)

        """
        Subscribe to a given topic. The topic must exist and the user must
        have sufficient permissions for that topic. Further the proxy object
        must match the required type for the topic as encoded in the topic
        name.
        Arguments:
        topicName -- 
        prx -- 
        _ctx -- The request context for the invocation.
        """
        def subscribe(self, topicName, prx, _ctx=None):
            return _M_omero.api.ServiceFactory._op_subscribe.invoke(self, ((topicName, prx), _ctx))

        """
        Subscribe to a given topic. The topic must exist and the user must
        have sufficient permissions for that topic. Further the proxy object
        must match the required type for the topic as encoded in the topic
        name.
        Arguments:
        topicName -- 
        prx -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_subscribe(self, topicName, prx, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_subscribe.begin(self, ((topicName, prx), _response, _ex, _sent, _ctx))

        """
        Subscribe to a given topic. The topic must exist and the user must
        have sufficient permissions for that topic. Further the proxy object
        must match the required type for the topic as encoded in the topic
        name.
        Arguments:
        topicName -- 
        prx -- 
        """
        def end_subscribe(self, _r):
            return _M_omero.api.ServiceFactory._op_subscribe.end(self, _r)

        """
        Sets the single callback used by the ServiceFactory
        to communicate with the client application. A default
        callback is set by the omero::client object on
        session creation which should suffice for most usage.
        See the client object's documentation in each language
        mapping for ways to use the callback.
        Arguments:
        callback -- 
        _ctx -- The request context for the invocation.
        """
        def setCallback(self, callback, _ctx=None):
            return _M_omero.api.ServiceFactory._op_setCallback.invoke(self, ((callback, ), _ctx))

        """
        Sets the single callback used by the ServiceFactory
        to communicate with the client application. A default
        callback is set by the omero::client object on
        session creation which should suffice for most usage.
        See the client object's documentation in each language
        mapping for ways to use the callback.
        Arguments:
        callback -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_setCallback(self, callback, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_setCallback.begin(self, ((callback, ), _response, _ex, _sent, _ctx))

        """
        Sets the single callback used by the ServiceFactory
        to communicate with the client application. A default
        callback is set by the omero::client object on
        session creation which should suffice for most usage.
        See the client object's documentation in each language
        mapping for ways to use the callback.
        Arguments:
        callback -- 
        """
        def end_setCallback(self, _r):
            return _M_omero.api.ServiceFactory._op_setCallback.end(self, _r)

        """
        Marks the session for closure rather than detachment, which will
        be triggered by the destruction of the Glacier2 connection via
        router.destroySession()
        Closing the session rather the detaching is more secure, since all
        resources are removed from the server and can safely be set once
        it is clear that a client is finished with those resources.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def closeOnDestroy(self, _ctx=None):
            return _M_omero.api.ServiceFactory._op_closeOnDestroy.invoke(self, ((), _ctx))

        """
        Marks the session for closure rather than detachment, which will
        be triggered by the destruction of the Glacier2 connection via
        router.destroySession()
        Closing the session rather the detaching is more secure, since all
        resources are removed from the server and can safely be set once
        it is clear that a client is finished with those resources.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_closeOnDestroy(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_closeOnDestroy.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Marks the session for closure rather than detachment, which will
        be triggered by the destruction of the Glacier2 connection via
        router.destroySession()
        Closing the session rather the detaching is more secure, since all
        resources are removed from the server and can safely be set once
        it is clear that a client is finished with those resources.
        Arguments:
        """
        def end_closeOnDestroy(self, _r):
            return _M_omero.api.ServiceFactory._op_closeOnDestroy.end(self, _r)

        """
        Marks the session for detachment rather than closure, which will
        be triggered by the destruction of the Glacier2 connection via
        router.destroySession()
        This is the default and allows a lost session to be reconnected,
        at a slight security cost since the session will persist longer
        and can be used by others if the UUID is intercepted.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def detachOnDestroy(self, _ctx=None):
            return _M_omero.api.ServiceFactory._op_detachOnDestroy.invoke(self, ((), _ctx))

        """
        Marks the session for detachment rather than closure, which will
        be triggered by the destruction of the Glacier2 connection via
        router.destroySession()
        This is the default and allows a lost session to be reconnected,
        at a slight security cost since the session will persist longer
        and can be used by others if the UUID is intercepted.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_detachOnDestroy(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_detachOnDestroy.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Marks the session for detachment rather than closure, which will
        be triggered by the destruction of the Glacier2 connection via
        router.destroySession()
        This is the default and allows a lost session to be reconnected,
        at a slight security cost since the session will persist longer
        and can be used by others if the UUID is intercepted.
        Arguments:
        """
        def end_detachOnDestroy(self, _r):
            return _M_omero.api.ServiceFactory._op_detachOnDestroy.end(self, _r)

        """
        Returns a list of string ids for currently active services. This will
        _not_ keep services alive, and in fact checks for all expired services
        and removes them.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def activeServices(self, _ctx=None):
            return _M_omero.api.ServiceFactory._op_activeServices.invoke(self, ((), _ctx))

        """
        Returns a list of string ids for currently active services. This will
        _not_ keep services alive, and in fact checks for all expired services
        and removes them.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_activeServices(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_activeServices.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Returns a list of string ids for currently active services. This will
        _not_ keep services alive, and in fact checks for all expired services
        and removes them.
        Arguments:
        """
        def end_activeServices(self, _r):
            return _M_omero.api.ServiceFactory._op_activeServices.end(self, _r)

        """
        Requests that the given services be marked as alive. It is
        possible that one of the services has already timed out, in which
        case the returned long value will be non-zero.
        Specifically, the bit representing the 0-based index will be 1:
        {@code
        if (retval & 1&lt;&lt;idx == 1&lt;&lt;idx) { // not alive }
        }
        Except for fatal server or session errors, this method should never
        throw an exception.
        Arguments:
        proxies -- 
        _ctx -- The request context for the invocation.
        """
        def keepAllAlive(self, proxies, _ctx=None):
            return _M_omero.api.ServiceFactory._op_keepAllAlive.invoke(self, ((proxies, ), _ctx))

        """
        Requests that the given services be marked as alive. It is
        possible that one of the services has already timed out, in which
        case the returned long value will be non-zero.
        Specifically, the bit representing the 0-based index will be 1:
        {@code
        if (retval & 1&lt;&lt;idx == 1&lt;&lt;idx) { // not alive }
        }
        Except for fatal server or session errors, this method should never
        throw an exception.
        Arguments:
        proxies -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_keepAllAlive(self, proxies, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_keepAllAlive.begin(self, ((proxies, ), _response, _ex, _sent, _ctx))

        """
        Requests that the given services be marked as alive. It is
        possible that one of the services has already timed out, in which
        case the returned long value will be non-zero.
        Specifically, the bit representing the 0-based index will be 1:
        {@code
        if (retval & 1&lt;&lt;idx == 1&lt;&lt;idx) { // not alive }
        }
        Except for fatal server or session errors, this method should never
        throw an exception.
        Arguments:
        proxies -- 
        """
        def end_keepAllAlive(self, _r):
            return _M_omero.api.ServiceFactory._op_keepAllAlive.end(self, _r)

        """
        Returns true if the given service is alive.
        Except for fatal server or session errors, this method should never
        throw an exception.
        Arguments:
        proxy -- 
        _ctx -- The request context for the invocation.
        """
        def keepAlive(self, proxy, _ctx=None):
            return _M_omero.api.ServiceFactory._op_keepAlive.invoke(self, ((proxy, ), _ctx))

        """
        Returns true if the given service is alive.
        Except for fatal server or session errors, this method should never
        throw an exception.
        Arguments:
        proxy -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_keepAlive(self, proxy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ServiceFactory._op_keepAlive.begin(self, ((proxy, ), _response, _ex, _sent, _ctx))

        """
        Returns true if the given service is alive.
        Except for fatal server or session errors, this method should never
        throw an exception.
        Arguments:
        proxy -- 
        """
        def end_keepAlive(self, _r):
            return _M_omero.api.ServiceFactory._op_keepAlive.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.api.ServiceFactoryPrx.ice_checkedCast(proxy, '::omero::api::ServiceFactory', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.api.ServiceFactoryPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::api::ServiceFactory'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.api._t_ServiceFactoryPrx = IcePy.defineProxy('::omero::api::ServiceFactory', ServiceFactoryPrx)

    _M_omero.api._t_ServiceFactory = IcePy.defineClass('::omero::api::ServiceFactory', ServiceFactory, -1, (), True, False, None, (_M_omero.cmd._t_Session,), ())
    ServiceFactory._ice_type = _M_omero.api._t_ServiceFactory

    ServiceFactory._op_getSecurityContexts = IcePy.Operation('getSecurityContexts', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.api._t_IObjectList, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_setSecurityContext = IcePy.Operation('setSecurityContext', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_IObject, False, 0),), (), ((), _M_omero.model._t_IObject, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_setSecurityPassword = IcePy.Operation('setSecurityPassword', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_string, False, 0),), (), None, (_M_omero._t_ServerError,))
    ServiceFactory._op_getAdminService = IcePy.Operation('getAdminService', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.api._t_IAdminPrx, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_getConfigService = IcePy.Operation('getConfigService', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.api._t_IConfigPrx, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_getContainerService = IcePy.Operation('getContainerService', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.api._t_IContainerPrx, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_getLdapService = IcePy.Operation('getLdapService', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.api._t_ILdapPrx, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_getPixelsService = IcePy.Operation('getPixelsService', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.api._t_IPixelsPrx, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_getProjectionService = IcePy.Operation('getProjectionService', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.api._t_IProjectionPrx, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_getQueryService = IcePy.Operation('getQueryService', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.api._t_IQueryPrx, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_getRenderingSettingsService = IcePy.Operation('getRenderingSettingsService', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.api._t_IRenderingSettingsPrx, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_getRepositoryInfoService = IcePy.Operation('getRepositoryInfoService', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.api._t_IRepositoryInfoPrx, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_getRoiService = IcePy.Operation('getRoiService', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.api._t_IRoiPrx, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_getScriptService = IcePy.Operation('getScriptService', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.api._t_IScriptPrx, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_getSessionService = IcePy.Operation('getSessionService', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.api._t_ISessionPrx, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_getShareService = IcePy.Operation('getShareService', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.api._t_ISharePrx, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_getTimelineService = IcePy.Operation('getTimelineService', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.api._t_ITimelinePrx, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_getTypesService = IcePy.Operation('getTypesService', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.api._t_ITypesPrx, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_getUpdateService = IcePy.Operation('getUpdateService', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.api._t_IUpdatePrx, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_getMetadataService = IcePy.Operation('getMetadataService', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.api._t_IMetadataPrx, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_createExporter = IcePy.Operation('createExporter', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.api._t_ExporterPrx, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_createJobHandle = IcePy.Operation('createJobHandle', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.api._t_JobHandlePrx, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_createRawFileStore = IcePy.Operation('createRawFileStore', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.api._t_RawFileStorePrx, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_createRawPixelsStore = IcePy.Operation('createRawPixelsStore', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.api._t_RawPixelsStorePrx, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_createRenderingEngine = IcePy.Operation('createRenderingEngine', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.api._t_RenderingEnginePrx, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_createSearchService = IcePy.Operation('createSearchService', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.api._t_SearchPrx, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_createThumbnailStore = IcePy.Operation('createThumbnailStore', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.api._t_ThumbnailStorePrx, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_sharedResources = IcePy.Operation('sharedResources', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.grid._t_SharedResourcesPrx, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_getByName = IcePy.Operation('getByName', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_string, False, 0),), (), ((), _M_omero.api._t_ServiceInterfacePrx, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_createByName = IcePy.Operation('createByName', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_string, False, 0),), (), ((), _M_omero.api._t_StatefulServiceInterfacePrx, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_subscribe = IcePy.Operation('subscribe', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_string, False, 0), ((), IcePy._t_ObjectPrx, False, 0)), (), None, (_M_omero._t_ServerError,))
    ServiceFactory._op_setCallback = IcePy.Operation('setCallback', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.api._t_ClientCallbackPrx, False, 0),), (), None, (_M_omero._t_ServerError,))
    ServiceFactory._op_closeOnDestroy = IcePy.Operation('closeOnDestroy', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, (_M_omero._t_ServerError,))
    ServiceFactory._op_detachOnDestroy = IcePy.Operation('detachOnDestroy', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, (_M_omero._t_ServerError,))
    ServiceFactory._op_activeServices = IcePy.Operation('activeServices', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.api._t_StringSet, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_keepAllAlive = IcePy.Operation('keepAllAlive', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.api._t_ServiceList, False, 0),), (), ((), IcePy._t_long, False, 0), (_M_omero._t_ServerError,))
    ServiceFactory._op_keepAlive = IcePy.Operation('keepAlive', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.api._t_ServiceInterfacePrx, False, 0),), (), ((), IcePy._t_bool, False, 0), (_M_omero._t_ServerError,))

    _M_omero.api.ServiceFactory = ServiceFactory
    del ServiceFactory

    _M_omero.api.ServiceFactoryPrx = ServiceFactoryPrx
    del ServiceFactoryPrx

# End of module omero.api

__name__ = 'omero'

# End of module omero



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# 
# Copyright 2008 Glencoe Software, Inc. All rights reserved.
# Use is subject to license terms supplied in LICENSE.txt
"""
load, quit, version, help plugins

Plugin read by omero.cli.Cli during initialization. The method(s)
defined here will be added to the Cli class for later use.

The load plugin is used to read in files with omero cli commands
(omitting the omero). For example,

./omero load some/file.osh

The help, quit, and version plugins are self-explanatory.
"""
from __future__ import print_function

from past.builtins import cmp
from glob import glob
import sys

from collections import defaultdict

from omero.cli import BaseControl
from omero.cli import CLI
from omero.cli import VERSION
from omero.cli import OMERODIR


class QuitControl(BaseControl):

    def _configure(self, parser):
        parser.set_defaults(func=self.__call__)

    def __call__(self, args):
        self.ctx.exit("", newline=False)


class VersionControl(BaseControl):

    def _configure(self, parser):
        parser.set_defaults(func=self.__call__)

    def __call__(self, args):
        self.ctx.err("OMERO.py version:")
        self.ctx.out(VERSION)
        server_version = None
        for line in self.ctx.get_config_property_lines(OMERODIR):
            line = str(line).strip()
            if line.startswith("omero.version="):
                server_version = line[len("omero.version="):]
        if server_version:
            self.ctx.err("OMERO.server version:")
            self.ctx.err(server_version)


LOAD_HELP = """Load file as if it were sent on standard in.

This can be used as the #! header of a file to make standand-alone script.

Examples:
    #!/usr/bin/env omero load
    login -C root@localhost
    group add new_group
    user add foo some user new_group

 or

    $ omero login       # login can't take place in HERE-document
    $ omero load <<EOF
    user list
    group list
    EOF

"""


class LoadControl(BaseControl):

    def _configure(self, parser):
        parser.add_argument("infile", nargs="*")
        parser.add_argument(
            "-g", "--glob", action="store_true", default=False,
            help=("Input paths are shell globs that should be expanded and "
                  "sorted."))
        parser.add_argument(
            "-k", "--keep-going", action="store_true", default=False,
            help="Continue processing after an error.")
        parser.set_defaults(func=self.__call__)

    def _load_filehandle(self, fh, keep_going):
        for line in fh:
            self.ctx.invoke(line, strict=(not keep_going))

            if self.ctx.rv != 0:
                self.ctx.err("Ignoring error: %s" % line)
                self.ctx.rv = 0

    def __call__(self, args):
        stdout = False
        if args.glob:
            infiles = []
            for fileglob in args.infile:
                infiles.extend(glob(fileglob))
            infiles.sort()
        elif not args.infile:
            stdout = True
        else:
            infiles = args.infile

        if stdout:
            self._load_filehandle(sys.stdin, args.keep_going)
        else:
            for filename in infiles:
                self.ctx.dbg("Loading file %s" % filename)
                with open(filename, 'r') as fh:
                    self._load_filehandle(fh, args.keep_going)


class ShellControl(BaseControl):

    def _configure(self, parser):
        parser.add_argument(
            "--login", action="store_true",
            help="Logins in and sets the 'client' variable")
        parser.add_argument("arg", nargs="*", help="Arguments for IPython.")
        parser.set_defaults(func=self.__call__)

    def __call__(self, args):
        """
        Copied from IPython embed-short example
        """
        import logging
        logging.basicConfig()
        from omero.util.upgrade_check import UpgradeCheck
        check = UpgradeCheck("shell")
        check.run()
        if check.isUpgradeNeeded():
            self.ctx.out("")

        ns = {}
        if args.login:
            import omero
            client = self.ctx.conn(args)
            ns = {"client": client, "omero": omero}

        try:
            # IPython 0.11 (see #7112)
            from IPython import embed
            embed(user_ns=ns)
        except ImportError:
            from IPython.Shell import IPShellEmbed
            ipshell = IPShellEmbed(args.arg)
            ipshell(local_ns=ns)

HELP_USAGE = """usage: %(program_name)s <command> [options] args
See 'help <command>' or '<command> -h' for more information on syntax
Type 'quit' to exit

Available commands:
%(commands)s

Other help topics:
%(topics)s

For additional information, see:
https://docs.openmicroscopy.org/latest/omero/users/cli/index.html
Report bugs at https://forum.image.sc/tag/omero-cli
"""


class HelpControl(BaseControl):
    """
    Defined here since the background loading might be too
    slow to have all help available
    """

    def _configure(self, parser):
        self.__parser__ = parser  # For formatting later
        parser.set_defaults(func=self.__call__)
        parser.add_argument(
            "--recursive", action="store_true",
            help="Also print help for all subcommands")
        group = parser.add_mutually_exclusive_group()
        group.add_argument(
            "--all", action="store_true",
            help="Print help for all commands and topics")
        group.add_argument(
            "--list", action="store_true",
            help="Print list of all commands and subcommands")
        group.add_argument(
            "topic", nargs="?", help="Command or topic for more information")

    def _complete(self, text, line, begidx, endidx):
        """
        This is something of a hack. This should either be a part
        of the context interface, or we should put it somewhere
        in a utility. FIXME.
        """
        return self.ctx.completenames(text, line, begidx, endidx)

    def format_title(self, command, sep="-"):
        """Create heading for command or topic help"""
        self.ctx.out("\n" + command)
        self.ctx.out(sep * len(command) + "\n")

    def print_command_help(self, control, args):
        """Print help for a single command and optionally its subcommand"""
        self.ctx.invoke([control, "-h"])
        if args.recursive:
            subcommands = self.ctx.controls[control].get_subcommands()
            for subcommand in subcommands:
                self.format_title(control + " " + subcommand, sep="^")
                self.ctx.invoke([control, subcommand, "-h"])

    def print_usage(self):
        commands, topics = [
            self.__parser__._format_list(x) for x in
            [sorted(self.ctx.controls), sorted(self.ctx.topics)]]
        key_list = {
            "program_name": sys.argv[0],
            "version": VERSION,
            "commands": commands,
            "topics": topics}
        print(HELP_USAGE % key_list)

    def print_single_command_or_topic(self, args):
        """Print the help for a command or a topic"""
        if args.topic in self.ctx.controls:
            self.print_command_help(args.topic, args)
        elif args.topic in self.ctx.topics:
            self.ctx.out(self.ctx.topics[args.topic])
        else:
            self.ctx.err("Unknown help topic or command: %s" % args.topic)

    def print_all_commands_and_topics(self, args):
        """Print the help for all commands and topics"""

        for control in sorted(self.ctx.controls):
            self.format_title(control)
            self.print_command_help(control, args)

        for topic in sorted(self.ctx.topics):
            self.format_title(topic)
            self.ctx.out(self.ctx.topics[topic])

    def print_commands_list(self, args):
        """Print a list of all commands"""

        for control in sorted(self.ctx.controls):
            subcommands = self.ctx.controls[control].get_subcommands()
            if subcommands:
                self.ctx.out("%s (%s)" % (control, len(subcommands)))
                for subcommand in subcommands:
                    self.ctx.out("\t%s" % subcommand)
            else:
                self.ctx.out("%s" % control)

    def __call__(self, args):

        self.ctx.waitForPlugins()

        # Fail-fast and print usage if no arg is passed
        if not args.all and not args.list and not args.topic:
            self.print_usage()

        if args.all:
            self.print_all_commands_and_topics(args)
        elif args.list:
            self.print_commands_list(args)
        elif args.topic:
            self.print_single_command_or_topic(args)


class ErrorsControl(BaseControl):

    def _configure(self, parser):
        parser.set_defaults(func=self.__call__)
        parser.add_argument("--length", default=50, type=int,
                            help="Length of message to print")
        parser.add_argument("plugins", nargs="*", default=(),
                            help="Limit to these plugins; otherwise all")

    def __call__(self, args):
        arranged = defaultdict(lambda: defaultdict(
            lambda: defaultdict(list)))
        for name, control in list(self.ctx.controls.items()):
            if not args.plugins or name in args.plugins:
                combined = []
                if hasattr(control, "get_errors"):
                    combined.extend(list(control.get_errors().items()))
                    combined.sort(key=lambda x: x[1].rcode)
                    for key, err in combined:
                        arranged[err.rcode][name][key].append(err)

        for rcode, names in sorted(arranged.items()):
            for name, keys in sorted(names.items()):
                for key, errors in sorted(keys.items()):
                    for err in errors:
                        msg = err.msg
                        if len(msg) > (args.length+1):
                            msg = msg[:args.length] + "..."
                        msg = msg.replace("\n", " ")
                        msg = msg.strip()
                        t = (err.rcode, name, key, msg)
                        self.ctx.out("%5d\t%10s\t%10s\t'%s'" % t)

controls = {
    "help": (HelpControl, "Syntax help for all commands"),
    "quit": (QuitControl, "Quit application"),
    "errors": (ErrorsControl, "Display all plugin error codes"),
    "shell": (ShellControl, """Starts an IPython interpreter session

All arguments not understood vi %(prog)s will be passed to the shell.
Use "--" to end parsing, e.g. '%(prog)s -- --help' for IPython help"""),
    "version": (VersionControl, "Version number"),
    "load": (LoadControl, LOAD_HELP)}

try:
    for k, v in list(controls.items()):
        register(k, v[0], v[1])
except NameError:
    if __name__ == "__main__":
        cli = CLI()
        for k, v in list(controls.items()):
            cli.register(k, v[0], v[1])
        cli.invoke(sys.argv[1:])



"""
   /*
   **   Generated by blitz/resources/templates/combined.vm
   **
   **   Copyright 2007, 2008 Glencoe Software, Inc. All rights reserved.
   **   Use is subject to license terms supplied in LICENSE.txt
   **
   */
"""
try:
   unicode
except NameError:
   # Python 3: "unicode" is built-in
   unicode = str
import Ice
import IceImport
import omero
IceImport.load("omero_model_DetailsI")
IceImport.load("omero_model_Image_ice")
from omero.rtypes import rlong
from collections import namedtuple
_omero = Ice.openModule("omero")
_omero_model = Ice.openModule("omero.model")
__name__ = "omero.model"
class ImageI(_omero_model.Image):

      # Property Metadata
      _field_info_data = namedtuple("FieldData", ["wrapper", "nullable"])
      _field_info_type = namedtuple("FieldInfo", [
          "series",
          "acquisitionDate",
          "archived",
          "partial",
          "format",
          "imagingEnvironment",
          "objectiveSettings",
          "instrument",
          "stageLabel",
          "experiment",
          "pixels",
          "wellSamples",
          "rois",
          "datasetLinks",
          "folderLinks",
          "fileset",
          "annotationLinks",
          "name",
          "description",
          "details",
      ])
      _field_info = _field_info_type(
          series=_field_info_data(wrapper=omero.rtypes.rint, nullable=True),
          acquisitionDate=_field_info_data(wrapper=omero.rtypes.rtime, nullable=True),
          archived=_field_info_data(wrapper=omero.rtypes.rbool, nullable=True),
          partial=_field_info_data(wrapper=omero.rtypes.rbool, nullable=True),
          format=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
          imagingEnvironment=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
          objectiveSettings=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
          instrument=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
          stageLabel=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
          experiment=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
          pixels=_field_info_data(wrapper=omero.proxy_to_instance, nullable=False),
          wellSamples=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
          rois=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
          datasetLinks=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
          folderLinks=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
          fileset=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
          annotationLinks=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
          name=_field_info_data(wrapper=omero.rtypes.rstring, nullable=False),
          description=_field_info_data(wrapper=omero.rtypes.rstring, nullable=True),
          details=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
      )  # end _field_info
      SERIES =  "ome.model.core.Image_series"
      ACQUISITIONDATE =  "ome.model.core.Image_acquisitionDate"
      ARCHIVED =  "ome.model.core.Image_archived"
      PARTIAL =  "ome.model.core.Image_partial"
      FORMAT =  "ome.model.core.Image_format"
      IMAGINGENVIRONMENT =  "ome.model.core.Image_imagingEnvironment"
      OBJECTIVESETTINGS =  "ome.model.core.Image_objectiveSettings"
      INSTRUMENT =  "ome.model.core.Image_instrument"
      STAGELABEL =  "ome.model.core.Image_stageLabel"
      EXPERIMENT =  "ome.model.core.Image_experiment"
      PIXELS =  "ome.model.core.Image_pixels"
      WELLSAMPLES =  "ome.model.core.Image_wellSamples"
      ROIS =  "ome.model.core.Image_rois"
      DATASETLINKS =  "ome.model.core.Image_datasetLinks"
      FOLDERLINKS =  "ome.model.core.Image_folderLinks"
      FILESET =  "ome.model.core.Image_fileset"
      ANNOTATIONLINKS =  "ome.model.core.Image_annotationLinks"
      NAME =  "ome.model.core.Image_name"
      DESCRIPTION =  "ome.model.core.Image_description"
      DETAILS =  "ome.model.core.Image_details"
      def errorIfUnloaded(self):
          if not self._loaded:
              raise _omero.UnloadedEntityException("Object unloaded:"+str(self))

      def throwNullCollectionException(self,propertyName):
          raise _omero.UnloadedEntityException(""+
          "Error updating collection:" + propertyName +"\n"+
          "Collection is currently null. This can be seen\n" +
          "by testing \""+ propertyName +"Loaded\". This implies\n"+
          "that this collection was unloaded. Please refresh this object\n"+
          "in order to update this collection.\n")

      def _toggleCollectionsLoaded(self, load):
          if load:
              self._pixelsSeq = []
              self._pixelsLoaded = True;
          else:
              self._pixelsSeq = []
              self._pixelsLoaded = False;

          if load:
              self._wellSamplesSeq = []
              self._wellSamplesLoaded = True;
          else:
              self._wellSamplesSeq = []
              self._wellSamplesLoaded = False;

          if load:
              self._roisSeq = []
              self._roisLoaded = True;
          else:
              self._roisSeq = []
              self._roisLoaded = False;

          if load:
              self._datasetLinksSeq = []
              self._datasetLinksLoaded = True;
          else:
              self._datasetLinksSeq = []
              self._datasetLinksLoaded = False;

          if load:
              self._folderLinksSeq = []
              self._folderLinksLoaded = True;
          else:
              self._folderLinksSeq = []
              self._folderLinksLoaded = False;

          if load:
              self._annotationLinksSeq = []
              self._annotationLinksLoaded = True;
          else:
              self._annotationLinksSeq = []
              self._annotationLinksLoaded = False;

          pass

      def __init__(self, id=None, loaded=None):
          super(ImageI, self).__init__()
          if id is not None and isinstance(id, (str, unicode)) and ":" in id:
              parts = id.split(":")
              if len(parts) != 2:
                  raise Exception("Invalid proxy string: %s", id)
              if parts[0] != self.__class__.__name__ and \
                 parts[0]+"I" != self.__class__.__name__:
                  raise Exception("Proxy class mismatch: %s<>%s" %
                  (self.__class__.__name__, parts[0]))
              self._id = rlong(parts[1])
              if loaded is None:
                  # If no loadedness was requested with
                  # a proxy string, then assume False.
                  loaded = False
          else:
              # Relying on omero.rtypes.rlong's error-handling
              self._id = rlong(id)
              if loaded is None:
                  loaded = True  # Assume true as previously
          self._loaded = loaded
          if self._loaded:
             self._details = _omero_model.DetailsI()
             self._toggleCollectionsLoaded(True)

      def unload(self, current = None):
          self._loaded = False
          self.unloadSeries( )
          self.unloadAcquisitionDate( )
          self.unloadArchived( )
          self.unloadPartial( )
          self.unloadFormat( )
          self.unloadImagingEnvironment( )
          self.unloadObjectiveSettings( )
          self.unloadInstrument( )
          self.unloadStageLabel( )
          self.unloadExperiment( )
          self.unloadPixels( )
          self.unloadWellSamples( )
          self.unloadRois( )
          self.unloadDatasetLinks( )
          self.unloadFolderLinks( )
          self.unloadFileset( )
          self.unloadAnnotationLinks( )
          self.unloadName( )
          self.unloadDescription( )
          self.unloadDetails( )

      def isLoaded(self, current = None):
          return self._loaded
      def unloadCollections(self, current = None):
          self._toggleCollectionsLoaded( False )
      def isGlobal(self, current = None):
          return  False ;
      def isMutable(self, current = None):
          return  True ;
      def isAnnotated(self, current = None):
          return  True ;
      def isLink(self, current = None):
          return  False ;
      def shallowCopy(self, current = None):
            if not self._loaded: return self.proxy()
            copy = ImageI()
            copy._id = self._id;
            copy._version = self._version;
            copy._details = None  # Unloading for the moment.
            raise omero.ClientError("NYI")
      def proxy(self, current = None):
          if self._id is None: raise omero.ClientError("Proxies require an id")
          return ImageI( self._id.getValue(), False )

      def getDetails(self, current = None):
          self.errorIfUnloaded()
          return self._details

      def unloadDetails(self, current = None):
          self._details = None

      def getId(self, current = None):
          return self._id

      def setId(self, _id, current = None):
          self._id = _id

      def checkUnloadedProperty(self, value, loadedField):
          if value == None:
              self.__dict__[loadedField] = False
          else:
              self.__dict__[loadedField] = True

      def getVersion(self, current = None):
          self.errorIfUnloaded()
          return self._version

      def setVersion(self, version, current = None):
          self.errorIfUnloaded()
          self._version = version

      def unloadSeries(self, ):
          self._seriesLoaded = False
          self._series = None;

      def getSeries(self, current = None):
          self.errorIfUnloaded()
          return self._series

      def setSeries(self, _series, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.series.wrapper is not None:
              if _series is not None:
                  _series = self._field_info.series.wrapper(_series)
          self._series = _series
          pass

      def unloadAcquisitionDate(self, ):
          self._acquisitionDateLoaded = False
          self._acquisitionDate = None;

      def getAcquisitionDate(self, current = None):
          self.errorIfUnloaded()
          return self._acquisitionDate

      def setAcquisitionDate(self, _acquisitionDate, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.acquisitionDate.wrapper is not None:
              if _acquisitionDate is not None:
                  _acquisitionDate = self._field_info.acquisitionDate.wrapper(_acquisitionDate)
          self._acquisitionDate = _acquisitionDate
          pass

      def unloadArchived(self, ):
          self._archivedLoaded = False
          self._archived = None;

      def getArchived(self, current = None):
          self.errorIfUnloaded()
          return self._archived

      def setArchived(self, _archived, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.archived.wrapper is not None:
              if _archived is not None:
                  _archived = self._field_info.archived.wrapper(_archived)
          self._archived = _archived
          pass

      def unloadPartial(self, ):
          self._partialLoaded = False
          self._partial = None;

      def getPartial(self, current = None):
          self.errorIfUnloaded()
          return self._partial

      def setPartial(self, _partial, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.partial.wrapper is not None:
              if _partial is not None:
                  _partial = self._field_info.partial.wrapper(_partial)
          self._partial = _partial
          pass

      def unloadFormat(self, ):
          self._formatLoaded = False
          self._format = None;

      def getFormat(self, current = None):
          self.errorIfUnloaded()
          return self._format

      def setFormat(self, _format, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.format.wrapper is not None:
              if _format is not None:
                  _format = self._field_info.format.wrapper(_format)
          self._format = _format
          pass

      def unloadImagingEnvironment(self, ):
          self._imagingEnvironmentLoaded = False
          self._imagingEnvironment = None;

      def getImagingEnvironment(self, current = None):
          self.errorIfUnloaded()
          return self._imagingEnvironment

      def setImagingEnvironment(self, _imagingEnvironment, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.imagingEnvironment.wrapper is not None:
              if _imagingEnvironment is not None:
                  _imagingEnvironment = self._field_info.imagingEnvironment.wrapper(_imagingEnvironment)
          self._imagingEnvironment = _imagingEnvironment
          pass

      def unloadObjectiveSettings(self, ):
          self._objectiveSettingsLoaded = False
          self._objectiveSettings = None;

      def getObjectiveSettings(self, current = None):
          self.errorIfUnloaded()
          return self._objectiveSettings

      def setObjectiveSettings(self, _objectiveSettings, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.objectiveSettings.wrapper is not None:
              if _objectiveSettings is not None:
                  _objectiveSettings = self._field_info.objectiveSettings.wrapper(_objectiveSettings)
          self._objectiveSettings = _objectiveSettings
          pass

      def unloadInstrument(self, ):
          self._instrumentLoaded = False
          self._instrument = None;

      def getInstrument(self, current = None):
          self.errorIfUnloaded()
          return self._instrument

      def setInstrument(self, _instrument, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.instrument.wrapper is not None:
              if _instrument is not None:
                  _instrument = self._field_info.instrument.wrapper(_instrument)
          self._instrument = _instrument
          pass

      def unloadStageLabel(self, ):
          self._stageLabelLoaded = False
          self._stageLabel = None;

      def getStageLabel(self, current = None):
          self.errorIfUnloaded()
          return self._stageLabel

      def setStageLabel(self, _stageLabel, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.stageLabel.wrapper is not None:
              if _stageLabel is not None:
                  _stageLabel = self._field_info.stageLabel.wrapper(_stageLabel)
          self._stageLabel = _stageLabel
          pass

      def unloadExperiment(self, ):
          self._experimentLoaded = False
          self._experiment = None;

      def getExperiment(self, current = None):
          self.errorIfUnloaded()
          return self._experiment

      def setExperiment(self, _experiment, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.experiment.wrapper is not None:
              if _experiment is not None:
                  _experiment = self._field_info.experiment.wrapper(_experiment)
          self._experiment = _experiment
          pass

      def unloadPixels(self, current = None):
          self._pixelsLoaded = False
          self._pixelsSeq = None;

      def _getPixels(self, current = None):
          self.errorIfUnloaded()
          return self._pixelsSeq

      def _setPixels(self, _pixels, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.pixelsSeq.wrapper is not None:
              if _pixels is not None:
                  _pixels = self._field_info.pixelsSeq.wrapper(_pixels)
          self._pixelsSeq = _pixels
          self.checkUnloadedProperty(_pixels,'pixelsLoaded')

      def isPixelsLoaded(self):
          return self._pixelsLoaded

      def sizeOfPixels(self, current = None):
          self.errorIfUnloaded()
          if not self._pixelsLoaded: return -1
          return len(self._pixelsSeq)

      def copyPixels(self, current = None):
          self.errorIfUnloaded()
          if not self._pixelsLoaded: self.throwNullCollectionException("pixelsSeq")
          return list(self._pixelsSeq)

      def iteratePixels(self):
          self.errorIfUnloaded()
          if not self._pixelsLoaded: self.throwNullCollectionException("pixelsSeq")
          return iter(self._pixelsSeq)

      def addPixels(self, target, current = None):
          self.errorIfUnloaded()
          if not self._pixelsLoaded: self.throwNullCollectionException("pixelsSeq")
          self._pixelsSeq.append( target );
          target.setImage( self )

      def addAllPixelsSet(self, targets, current = None):
          self.errorIfUnloaded()
          if  not self._pixelsLoaded: self.throwNullCollectionException("pixelsSeq")
          self._pixelsSeq.extend( targets )
          for target in targets:
              target.setImage( self )

      def removePixels(self, target, current = None):
          self.errorIfUnloaded()
          if not self._pixelsLoaded: self.throwNullCollectionException("pixelsSeq")
          self._pixelsSeq.remove( target )
          target.setImage( None )

      def removeAllPixelsSet(self, targets, current = None):
          self.errorIfUnloaded()
          if not self._pixelsLoaded: self.throwNullCollectionException("pixelsSeq")
          for elt in targets:
              elt.setImage( None )
              self._pixelsSeq.remove( elt )

      def clearPixels(self, current = None):
          self.errorIfUnloaded()
          if not self._pixelsLoaded: self.throwNullCollectionException("pixelsSeq")
          for elt in self._pixelsSeq:
              elt.setImage( None )
          self._pixelsSeq = list()

      def reloadPixels(self, toCopy, current = None):
          self.errorIfUnloaded()
          if self._pixelsLoaded:
              raise omero.ClientError("Cannot reload active collection: pixelsSeq")
          if not toCopy:
              raise omero.ClientError("Argument cannot be null")
          if toCopy.getId().getValue() != self.getId().getValue():
             raise omero.ClientError("Argument must have the same id as this instance")
          if toCopy.getDetails().getUpdateEvent().getId().getValue() < self.getDetails().getUpdateEvent().getId().getValue():
             raise omero.ClientError("Argument may not be older than this instance")
          copy = toCopy.copyPixels() # May also throw
          for elt in copy:
              elt.setImage( self )
          self._pixelsSeq = copy
          toCopy.unloadPixels()
          self._pixelsLoaded = True

      def getPixels(self, index, current = None):
          self.errorIfUnloaded()
          if not self._pixelsLoaded: self.throwNullCollectionException("pixelsSeq")
          return self._pixelsSeq[index]

      def setPixels(self, index, element, current = None, wrap=False):
          self.errorIfUnloaded()
          if not self._pixelsLoaded: self.throwNullCollectionException("pixelsSeq")
          old = self._pixelsSeq[index]
          if wrap and self._field_info.pixelsSeq.wrapper is not None:
              if element is not None:
                  element = self._field_info.pixelsSeq.wrapper(_pixels)
          self._pixelsSeq[index] =  element
          if element is not None and element.isLoaded():
              element.setImage( self )
          return old

      def getPrimaryPixels(self, current = None):
          self.errorIfUnloaded()
          if not self._pixelsLoaded: self.throwNullCollectionException("pixelsSeq")
          return self._pixelsSeq[0]

      def setPrimaryPixels(self, element, current = None):
          self.errorIfUnloaded()
          if not self._pixelsLoaded: self.throwNullCollectionException("pixelsSeq")
          index = self._pixelsSeq.index(element)
          old = self._pixelsSeq[0]
          self._pixelsSeq[index] = old
          self._pixelsSeq[0] = element
          return old

      def unloadWellSamples(self, current = None):
          self._wellSamplesLoaded = False
          self._wellSamplesSeq = None;

      def _getWellSamples(self, current = None):
          self.errorIfUnloaded()
          return self._wellSamplesSeq

      def _setWellSamples(self, _wellSamples, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.wellSamplesSeq.wrapper is not None:
              if _wellSamples is not None:
                  _wellSamples = self._field_info.wellSamplesSeq.wrapper(_wellSamples)
          self._wellSamplesSeq = _wellSamples
          self.checkUnloadedProperty(_wellSamples,'wellSamplesLoaded')

      def isWellSamplesLoaded(self):
          return self._wellSamplesLoaded

      def sizeOfWellSamples(self, current = None):
          self.errorIfUnloaded()
          if not self._wellSamplesLoaded: return -1
          return len(self._wellSamplesSeq)

      def copyWellSamples(self, current = None):
          self.errorIfUnloaded()
          if not self._wellSamplesLoaded: self.throwNullCollectionException("wellSamplesSeq")
          return list(self._wellSamplesSeq)

      def iterateWellSamples(self):
          self.errorIfUnloaded()
          if not self._wellSamplesLoaded: self.throwNullCollectionException("wellSamplesSeq")
          return iter(self._wellSamplesSeq)

      def addWellSample(self, target, current = None):
          self.errorIfUnloaded()
          if not self._wellSamplesLoaded: self.throwNullCollectionException("wellSamplesSeq")
          self._wellSamplesSeq.append( target );
          target.setImage( self )

      def addAllWellSampleSet(self, targets, current = None):
          self.errorIfUnloaded()
          if  not self._wellSamplesLoaded: self.throwNullCollectionException("wellSamplesSeq")
          self._wellSamplesSeq.extend( targets )
          for target in targets:
              target.setImage( self )

      def removeWellSample(self, target, current = None):
          self.errorIfUnloaded()
          if not self._wellSamplesLoaded: self.throwNullCollectionException("wellSamplesSeq")
          self._wellSamplesSeq.remove( target )
          target.setImage( None )

      def removeAllWellSampleSet(self, targets, current = None):
          self.errorIfUnloaded()
          if not self._wellSamplesLoaded: self.throwNullCollectionException("wellSamplesSeq")
          for elt in targets:
              elt.setImage( None )
              self._wellSamplesSeq.remove( elt )

      def clearWellSamples(self, current = None):
          self.errorIfUnloaded()
          if not self._wellSamplesLoaded: self.throwNullCollectionException("wellSamplesSeq")
          for elt in self._wellSamplesSeq:
              elt.setImage( None )
          self._wellSamplesSeq = list()

      def reloadWellSamples(self, toCopy, current = None):
          self.errorIfUnloaded()
          if self._wellSamplesLoaded:
              raise omero.ClientError("Cannot reload active collection: wellSamplesSeq")
          if not toCopy:
              raise omero.ClientError("Argument cannot be null")
          if toCopy.getId().getValue() != self.getId().getValue():
             raise omero.ClientError("Argument must have the same id as this instance")
          if toCopy.getDetails().getUpdateEvent().getId().getValue() < self.getDetails().getUpdateEvent().getId().getValue():
             raise omero.ClientError("Argument may not be older than this instance")
          copy = toCopy.copyWellSamples() # May also throw
          for elt in copy:
              elt.setImage( self )
          self._wellSamplesSeq = copy
          toCopy.unloadWellSamples()
          self._wellSamplesLoaded = True

      def unloadRois(self, current = None):
          self._roisLoaded = False
          self._roisSeq = None;

      def _getRois(self, current = None):
          self.errorIfUnloaded()
          return self._roisSeq

      def _setRois(self, _rois, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.roisSeq.wrapper is not None:
              if _rois is not None:
                  _rois = self._field_info.roisSeq.wrapper(_rois)
          self._roisSeq = _rois
          self.checkUnloadedProperty(_rois,'roisLoaded')

      def isRoisLoaded(self):
          return self._roisLoaded

      def sizeOfRois(self, current = None):
          self.errorIfUnloaded()
          if not self._roisLoaded: return -1
          return len(self._roisSeq)

      def copyRois(self, current = None):
          self.errorIfUnloaded()
          if not self._roisLoaded: self.throwNullCollectionException("roisSeq")
          return list(self._roisSeq)

      def iterateRois(self):
          self.errorIfUnloaded()
          if not self._roisLoaded: self.throwNullCollectionException("roisSeq")
          return iter(self._roisSeq)

      def addRoi(self, target, current = None):
          self.errorIfUnloaded()
          if not self._roisLoaded: self.throwNullCollectionException("roisSeq")
          self._roisSeq.append( target );
          target.setImage( self )

      def addAllRoiSet(self, targets, current = None):
          self.errorIfUnloaded()
          if  not self._roisLoaded: self.throwNullCollectionException("roisSeq")
          self._roisSeq.extend( targets )
          for target in targets:
              target.setImage( self )

      def removeRoi(self, target, current = None):
          self.errorIfUnloaded()
          if not self._roisLoaded: self.throwNullCollectionException("roisSeq")
          self._roisSeq.remove( target )
          target.setImage( None )

      def removeAllRoiSet(self, targets, current = None):
          self.errorIfUnloaded()
          if not self._roisLoaded: self.throwNullCollectionException("roisSeq")
          for elt in targets:
              elt.setImage( None )
              self._roisSeq.remove( elt )

      def clearRois(self, current = None):
          self.errorIfUnloaded()
          if not self._roisLoaded: self.throwNullCollectionException("roisSeq")
          for elt in self._roisSeq:
              elt.setImage( None )
          self._roisSeq = list()

      def reloadRois(self, toCopy, current = None):
          self.errorIfUnloaded()
          if self._roisLoaded:
              raise omero.ClientError("Cannot reload active collection: roisSeq")
          if not toCopy:
              raise omero.ClientError("Argument cannot be null")
          if toCopy.getId().getValue() != self.getId().getValue():
             raise omero.ClientError("Argument must have the same id as this instance")
          if toCopy.getDetails().getUpdateEvent().getId().getValue() < self.getDetails().getUpdateEvent().getId().getValue():
             raise omero.ClientError("Argument may not be older than this instance")
          copy = toCopy.copyRois() # May also throw
          for elt in copy:
              elt.setImage( self )
          self._roisSeq = copy
          toCopy.unloadRois()
          self._roisLoaded = True

      def unloadDatasetLinks(self, current = None):
          self._datasetLinksLoaded = False
          self._datasetLinksSeq = None;

      def _getDatasetLinks(self, current = None):
          self.errorIfUnloaded()
          return self._datasetLinksSeq

      def _setDatasetLinks(self, _datasetLinks, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.datasetLinksSeq.wrapper is not None:
              if _datasetLinks is not None:
                  _datasetLinks = self._field_info.datasetLinksSeq.wrapper(_datasetLinks)
          self._datasetLinksSeq = _datasetLinks
          self.checkUnloadedProperty(_datasetLinks,'datasetLinksLoaded')

      def isDatasetLinksLoaded(self):
          return self._datasetLinksLoaded

      def sizeOfDatasetLinks(self, current = None):
          self.errorIfUnloaded()
          if not self._datasetLinksLoaded: return -1
          return len(self._datasetLinksSeq)

      def copyDatasetLinks(self, current = None):
          self.errorIfUnloaded()
          if not self._datasetLinksLoaded: self.throwNullCollectionException("datasetLinksSeq")
          return list(self._datasetLinksSeq)

      def iterateDatasetLinks(self):
          self.errorIfUnloaded()
          if not self._datasetLinksLoaded: self.throwNullCollectionException("datasetLinksSeq")
          return iter(self._datasetLinksSeq)

      def addDatasetImageLink(self, target, current = None):
          self.errorIfUnloaded()
          if not self._datasetLinksLoaded: self.throwNullCollectionException("datasetLinksSeq")
          self._datasetLinksSeq.append( target );
          target.setChild( self )

      def addAllDatasetImageLinkSet(self, targets, current = None):
          self.errorIfUnloaded()
          if  not self._datasetLinksLoaded: self.throwNullCollectionException("datasetLinksSeq")
          self._datasetLinksSeq.extend( targets )
          for target in targets:
              target.setChild( self )

      def removeDatasetImageLink(self, target, current = None):
          self.errorIfUnloaded()
          if not self._datasetLinksLoaded: self.throwNullCollectionException("datasetLinksSeq")
          self._datasetLinksSeq.remove( target )
          target.setChild( None )

      def removeAllDatasetImageLinkSet(self, targets, current = None):
          self.errorIfUnloaded()
          if not self._datasetLinksLoaded: self.throwNullCollectionException("datasetLinksSeq")
          for elt in targets:
              elt.setChild( None )
              self._datasetLinksSeq.remove( elt )

      def clearDatasetLinks(self, current = None):
          self.errorIfUnloaded()
          if not self._datasetLinksLoaded: self.throwNullCollectionException("datasetLinksSeq")
          for elt in self._datasetLinksSeq:
              elt.setChild( None )
          self._datasetLinksSeq = list()

      def reloadDatasetLinks(self, toCopy, current = None):
          self.errorIfUnloaded()
          if self._datasetLinksLoaded:
              raise omero.ClientError("Cannot reload active collection: datasetLinksSeq")
          if not toCopy:
              raise omero.ClientError("Argument cannot be null")
          if toCopy.getId().getValue() != self.getId().getValue():
             raise omero.ClientError("Argument must have the same id as this instance")
          if toCopy.getDetails().getUpdateEvent().getId().getValue() < self.getDetails().getUpdateEvent().getId().getValue():
             raise omero.ClientError("Argument may not be older than this instance")
          copy = toCopy.copyDatasetLinks() # May also throw
          for elt in copy:
              elt.setChild( self )
          self._datasetLinksSeq = copy
          toCopy.unloadDatasetLinks()
          self._datasetLinksLoaded = True

      def getDatasetLinksCountPerOwner(self, current = None):
          return self._datasetLinksCountPerOwner

      def linkDataset(self, addition, current = None):
          self.errorIfUnloaded()
          if not self._datasetLinksLoaded: self.throwNullCollectionException("datasetLinksSeq")
          link = _omero_model.DatasetImageLinkI()
          link.link( addition, self );
          self.addDatasetImageLinkToBoth( link, True )
          return link

      def addDatasetImageLinkToBoth(self, link, bothSides):
          self.errorIfUnloaded()
          if not self._datasetLinksLoaded: self.throwNullCollectionException("datasetLinksSeq")
          self._datasetLinksSeq.append( link )
          if bothSides and link.getParent().isLoaded():
              link.getParent().addDatasetImageLinkToBoth( link, False )

      def findDatasetImageLink(self, removal, current = None):
          self.errorIfUnloaded()
          if not self._datasetLinksLoaded: self.throwNullCollectionException("datasetLinksSeq")
          result = list()
          for link in self._datasetLinksSeq:
              if link.getParent() == removal: result.append(link)
          return result

      def unlinkDataset(self, removal, current = None):
          self.errorIfUnloaded()
          if not self._datasetLinksLoaded: self.throwNullCollectionException("datasetLinksSeq")
          toRemove = self.findDatasetImageLink(removal)
          for next in toRemove:
              self.removeDatasetImageLinkFromBoth( next, True )

      def removeDatasetImageLinkFromBoth(self, link, bothSides, current = None):
          self.errorIfUnloaded()
          if not self._datasetLinksLoaded: self.throwNullCollectionException("datasetLinksSeq")
          self._datasetLinksSeq.remove( link )
          if bothSides and link.getParent().isLoaded():
              link.getParent().removeDatasetImageLinkFromBoth(link, False)

      def linkedDatasetList(self, current = None):
          self.errorIfUnloaded()
          if not self.datasetLinksLoaded: self.throwNullCollectionException("DatasetLinks")
          linked = []
          for link in self._datasetLinksSeq:
              linked.append( link.getParent() )
          return linked

      def unloadFolderLinks(self, current = None):
          self._folderLinksLoaded = False
          self._folderLinksSeq = None;

      def _getFolderLinks(self, current = None):
          self.errorIfUnloaded()
          return self._folderLinksSeq

      def _setFolderLinks(self, _folderLinks, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.folderLinksSeq.wrapper is not None:
              if _folderLinks is not None:
                  _folderLinks = self._field_info.folderLinksSeq.wrapper(_folderLinks)
          self._folderLinksSeq = _folderLinks
          self.checkUnloadedProperty(_folderLinks,'folderLinksLoaded')

      def isFolderLinksLoaded(self):
          return self._folderLinksLoaded

      def sizeOfFolderLinks(self, current = None):
          self.errorIfUnloaded()
          if not self._folderLinksLoaded: return -1
          return len(self._folderLinksSeq)

      def copyFolderLinks(self, current = None):
          self.errorIfUnloaded()
          if not self._folderLinksLoaded: self.throwNullCollectionException("folderLinksSeq")
          return list(self._folderLinksSeq)

      def iterateFolderLinks(self):
          self.errorIfUnloaded()
          if not self._folderLinksLoaded: self.throwNullCollectionException("folderLinksSeq")
          return iter(self._folderLinksSeq)

      def addFolderImageLink(self, target, current = None):
          self.errorIfUnloaded()
          if not self._folderLinksLoaded: self.throwNullCollectionException("folderLinksSeq")
          self._folderLinksSeq.append( target );
          target.setChild( self )

      def addAllFolderImageLinkSet(self, targets, current = None):
          self.errorIfUnloaded()
          if  not self._folderLinksLoaded: self.throwNullCollectionException("folderLinksSeq")
          self._folderLinksSeq.extend( targets )
          for target in targets:
              target.setChild( self )

      def removeFolderImageLink(self, target, current = None):
          self.errorIfUnloaded()
          if not self._folderLinksLoaded: self.throwNullCollectionException("folderLinksSeq")
          self._folderLinksSeq.remove( target )
          target.setChild( None )

      def removeAllFolderImageLinkSet(self, targets, current = None):
          self.errorIfUnloaded()
          if not self._folderLinksLoaded: self.throwNullCollectionException("folderLinksSeq")
          for elt in targets:
              elt.setChild( None )
              self._folderLinksSeq.remove( elt )

      def clearFolderLinks(self, current = None):
          self.errorIfUnloaded()
          if not self._folderLinksLoaded: self.throwNullCollectionException("folderLinksSeq")
          for elt in self._folderLinksSeq:
              elt.setChild( None )
          self._folderLinksSeq = list()

      def reloadFolderLinks(self, toCopy, current = None):
          self.errorIfUnloaded()
          if self._folderLinksLoaded:
              raise omero.ClientError("Cannot reload active collection: folderLinksSeq")
          if not toCopy:
              raise omero.ClientError("Argument cannot be null")
          if toCopy.getId().getValue() != self.getId().getValue():
             raise omero.ClientError("Argument must have the same id as this instance")
          if toCopy.getDetails().getUpdateEvent().getId().getValue() < self.getDetails().getUpdateEvent().getId().getValue():
             raise omero.ClientError("Argument may not be older than this instance")
          copy = toCopy.copyFolderLinks() # May also throw
          for elt in copy:
              elt.setChild( self )
          self._folderLinksSeq = copy
          toCopy.unloadFolderLinks()
          self._folderLinksLoaded = True

      def getFolderLinksCountPerOwner(self, current = None):
          return self._folderLinksCountPerOwner

      def linkFolder(self, addition, current = None):
          self.errorIfUnloaded()
          if not self._folderLinksLoaded: self.throwNullCollectionException("folderLinksSeq")
          link = _omero_model.FolderImageLinkI()
          link.link( addition, self );
          self.addFolderImageLinkToBoth( link, True )
          return link

      def addFolderImageLinkToBoth(self, link, bothSides):
          self.errorIfUnloaded()
          if not self._folderLinksLoaded: self.throwNullCollectionException("folderLinksSeq")
          self._folderLinksSeq.append( link )
          if bothSides and link.getParent().isLoaded():
              link.getParent().addFolderImageLinkToBoth( link, False )

      def findFolderImageLink(self, removal, current = None):
          self.errorIfUnloaded()
          if not self._folderLinksLoaded: self.throwNullCollectionException("folderLinksSeq")
          result = list()
          for link in self._folderLinksSeq:
              if link.getParent() == removal: result.append(link)
          return result

      def unlinkFolder(self, removal, current = None):
          self.errorIfUnloaded()
          if not self._folderLinksLoaded: self.throwNullCollectionException("folderLinksSeq")
          toRemove = self.findFolderImageLink(removal)
          for next in toRemove:
              self.removeFolderImageLinkFromBoth( next, True )

      def removeFolderImageLinkFromBoth(self, link, bothSides, current = None):
          self.errorIfUnloaded()
          if not self._folderLinksLoaded: self.throwNullCollectionException("folderLinksSeq")
          self._folderLinksSeq.remove( link )
          if bothSides and link.getParent().isLoaded():
              link.getParent().removeFolderImageLinkFromBoth(link, False)

      def linkedFolderList(self, current = None):
          self.errorIfUnloaded()
          if not self.folderLinksLoaded: self.throwNullCollectionException("FolderLinks")
          linked = []
          for link in self._folderLinksSeq:
              linked.append( link.getParent() )
          return linked

      def unloadFileset(self, ):
          self._filesetLoaded = False
          self._fileset = None;

      def getFileset(self, current = None):
          self.errorIfUnloaded()
          return self._fileset

      def setFileset(self, _fileset, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.fileset.wrapper is not None:
              if _fileset is not None:
                  _fileset = self._field_info.fileset.wrapper(_fileset)
          self._fileset = _fileset
          pass

      def unloadAnnotationLinks(self, current = None):
          self._annotationLinksLoaded = False
          self._annotationLinksSeq = None;

      def _getAnnotationLinks(self, current = None):
          self.errorIfUnloaded()
          return self._annotationLinksSeq

      def _setAnnotationLinks(self, _annotationLinks, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.annotationLinksSeq.wrapper is not None:
              if _annotationLinks is not None:
                  _annotationLinks = self._field_info.annotationLinksSeq.wrapper(_annotationLinks)
          self._annotationLinksSeq = _annotationLinks
          self.checkUnloadedProperty(_annotationLinks,'annotationLinksLoaded')

      def isAnnotationLinksLoaded(self):
          return self._annotationLinksLoaded

      def sizeOfAnnotationLinks(self, current = None):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: return -1
          return len(self._annotationLinksSeq)

      def copyAnnotationLinks(self, current = None):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          return list(self._annotationLinksSeq)

      def iterateAnnotationLinks(self):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          return iter(self._annotationLinksSeq)

      def addImageAnnotationLink(self, target, current = None):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          self._annotationLinksSeq.append( target );
          target.setParent( self )

      def addAllImageAnnotationLinkSet(self, targets, current = None):
          self.errorIfUnloaded()
          if  not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          self._annotationLinksSeq.extend( targets )
          for target in targets:
              target.setParent( self )

      def removeImageAnnotationLink(self, target, current = None):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          self._annotationLinksSeq.remove( target )
          target.setParent( None )

      def removeAllImageAnnotationLinkSet(self, targets, current = None):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          for elt in targets:
              elt.setParent( None )
              self._annotationLinksSeq.remove( elt )

      def clearAnnotationLinks(self, current = None):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          for elt in self._annotationLinksSeq:
              elt.setParent( None )
          self._annotationLinksSeq = list()

      def reloadAnnotationLinks(self, toCopy, current = None):
          self.errorIfUnloaded()
          if self._annotationLinksLoaded:
              raise omero.ClientError("Cannot reload active collection: annotationLinksSeq")
          if not toCopy:
              raise omero.ClientError("Argument cannot be null")
          if toCopy.getId().getValue() != self.getId().getValue():
             raise omero.ClientError("Argument must have the same id as this instance")
          if toCopy.getDetails().getUpdateEvent().getId().getValue() < self.getDetails().getUpdateEvent().getId().getValue():
             raise omero.ClientError("Argument may not be older than this instance")
          copy = toCopy.copyAnnotationLinks() # May also throw
          for elt in copy:
              elt.setParent( self )
          self._annotationLinksSeq = copy
          toCopy.unloadAnnotationLinks()
          self._annotationLinksLoaded = True

      def getAnnotationLinksCountPerOwner(self, current = None):
          return self._annotationLinksCountPerOwner

      def linkAnnotation(self, addition, current = None):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          link = _omero_model.ImageAnnotationLinkI()
          link.link( self, addition );
          self.addImageAnnotationLinkToBoth( link, True )
          return link

      def addImageAnnotationLinkToBoth(self, link, bothSides):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          self._annotationLinksSeq.append( link )

      def findImageAnnotationLink(self, removal, current = None):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          result = list()
          for link in self._annotationLinksSeq:
              if link.getChild() == removal: result.append(link)
          return result

      def unlinkAnnotation(self, removal, current = None):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          toRemove = self.findImageAnnotationLink(removal)
          for next in toRemove:
              self.removeImageAnnotationLinkFromBoth( next, True )

      def removeImageAnnotationLinkFromBoth(self, link, bothSides, current = None):
          self.errorIfUnloaded()
          if not self._annotationLinksLoaded: self.throwNullCollectionException("annotationLinksSeq")
          self._annotationLinksSeq.remove( link )

      def linkedAnnotationList(self, current = None):
          self.errorIfUnloaded()
          if not self.annotationLinksLoaded: self.throwNullCollectionException("AnnotationLinks")
          linked = []
          for link in self._annotationLinksSeq:
              linked.append( link.getChild() )
          return linked

      def unloadName(self, ):
          self._nameLoaded = False
          self._name = None;

      def getName(self, current = None):
          self.errorIfUnloaded()
          return self._name

      def setName(self, _name, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.name.wrapper is not None:
              if _name is not None:
                  _name = self._field_info.name.wrapper(_name)
          self._name = _name
          pass

      def unloadDescription(self, ):
          self._descriptionLoaded = False
          self._description = None;

      def getDescription(self, current = None):
          self.errorIfUnloaded()
          return self._description

      def setDescription(self, _description, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.description.wrapper is not None:
              if _description is not None:
                  _description = self._field_info.description.wrapper(_description)
          self._description = _description
          pass


      def ice_postUnmarshal(self):
          """
          Provides additional initialization once all data loaded
          """
          pass # Currently unused


      def ice_preMarshal(self):
          """
          Provides additional validation before data is sent
          """
          pass # Currently unused

      def __getattr__(self, name):
          """
          Reroutes all access to object.field through object.getField() or object.isField()
          """
          if "_" in name:  # Ice disallows underscores, so these should be treated normally.
              return object.__getattribute__(self, name)
          field  = "_" + name
          capitalized = name[0].capitalize() + name[1:]
          getter = "get" + capitalized
          questn = "is" + capitalized
          try:
              self.__dict__[field]
              if hasattr(self, getter):
                  method = getattr(self, getter)
                  return method()
              elif hasattr(self, questn):
                  method = getattr(self, questn)
                  return method()
          except:
              pass
          raise AttributeError("'%s' object has no attribute '%s' or '%s'" % (self.__class__.__name__, getter, questn))

      def __setattr__(self, name, value):
          """
          Reroutes all access to object.field through object.getField(), with the caveat
          that all sets on variables starting with "_" are permitted directly.
          """
          if name.startswith("_"):
              self.__dict__[name] = value
              return
          else:
              field  = "_" + name
              setter = "set" + name[0].capitalize() + name[1:]
              if hasattr(self, field) and hasattr(self, setter):
                  method = getattr(self, setter)
                  return method(value)
          raise AttributeError("'%s' object has no attribute '%s'" % (self.__class__.__name__, setter))

_omero_model.ImageI = ImageI



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# Copyright (C) 2010-2014 Glencoe Software, Inc.
# All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""
Library for managing user sessions.
"""
from __future__ import division
from __future__ import print_function

from future import standard_library
standard_library.install_aliases()
from builtins import str
from builtins import object
from past.utils import old_div
import omero.constants
from omero.util import get_omero_userdir, make_logname
from omero.rtypes import rlong
from omero_ext.path import path

try:
    from urllib.parse import quote, unquote
except ImportError:
    # Python2
    from urllib.parse import quote, unquote

import logging

"""
 * Track last used
 * provide single library (with lock) which does all of this
   - save session
   - clear session
   - check session # detachOnDestroy
   - list previous sessions
 * Use an environment variable for changing directories

import subprocess, optparse, os, sys
import getpass, pickle
import omero.java
from omero.cli import Arguments, BaseControl, VERSION
from path import path

"""


def _escape_host(host):
    return quote(host, safe='')


class SessionsStore(object):

    """
    The store is a file-based repository of user sessions.
    By default, stores use $HOME/omero/sessions as their
    repository path.

    Use add() to add items to the repository
    """

    def __init__(self, dir=None):
        """
        """
        self.logger = logging.getLogger(make_logname(self))
        if dir is None:
            self.dir = old_div(get_omero_userdir(), "sessions")
        else:
            self.dir = path(dir)
        if not self.dir.exists():
            self.dir.makedirs()
        try:
            self.dir.chmod(0o700)
        except:
            print("WARN: failed to chmod %s" % self.dir)

    #
    # File-only methods
    #

    def report(self):
        """
        Simple dump utility
        """
        for host in self.dir.dirs():
            print("[%s]" % host)
            for name in host.dirs():
                print(" -> %s : " % name)
                for sess in name.files():
                    print("    %s" % sess)

    def add(self, host, name, id, props, sudo=None):
        """
        Stores a file containing the properties at
        REPO/host/name/id
        """

        props["omero.host"] = host
        props["omero.user"] = name
        props["omero.sess"] = id
        if sudo is not None:
            props["omero.sudo"] = sudo

        lines = []
        for k, v in list(props.items()):
            lines.append("%s=%s" % (k, v))

        dhn = self.dir / _escape_host(host) / name
        if not dhn.exists():
            dhn.makedirs()

        (old_div(dhn, id)).write_lines(lines)

    def conflicts(self, host, name, id, new_props, ignore_nulls=False,
                  check_group=True):
        """
        Compares if the passed properties are compatible with
        with those for the host, name, id tuple

        If ignore_nulls is True, then a null in new_props means matches
        anything.
        """
        conflicts = ""
        old_props = self.get(host, name, id)
        default_port = str(omero.constants.GLACIER2PORT)
        keys = ["omero.port"]
        if check_group:
            keys.append("omero.group")

        for key in keys:
            old = old_props.get(key, None)
            new = new_props.get(key, None)
            if ignore_nulls and new is None:
                continue
            elif (key == "omero.port" and
                  set((old, new)) == set((None, default_port))):
                continue
            elif old != new:
                if conflicts != "":
                    conflicts += "; "
                conflicts += "%s: %s!=%s" % (key, old, new)
        return conflicts

    def remove(self, host, name, uuid):
        """
        Removes the given session file from the store
        and removes the sess_file() if it is equal to
        the session we just removed.
        """
        if uuid is None:
            self.logger.debug("No uuid provided")
            return
        d = self.dir / _escape_host(host) / name
        if d.exists():
            f = old_div(d, uuid)
            if f.exists():
                f.remove()
                self.logger.debug("Removed %s" % f)
            s = self.sess_file(host, name)
            if s and s.exists() and s.text().strip() == uuid:
                s.remove()
                self.logger.debug("Removed %s" % s)

    def exists(self, host, name, uuid):
        """
        Checks if the given file exists.
        """
        d = self.dir
        for x in (_escape_host(host), name, uuid):
            d = old_div(d, x)
            if not d.exists():
                return False
        return True

    def get(self, host, name, uuid):
        """
        Returns the properties stored in the given session file
        """
        return self.props(self.dir / _escape_host(host) / name / uuid)

    def available(self, host, name):
        """
        Returns the path to property files which are stored.
        Internal accounting files are not returned.
        """
        d = self.dir / _escape_host(host) / name
        if not d.exists():
            return []
        return [x.basename() for x in self.non_dot(d)]

    def set_current(self, host, name=None, uuid=None, props=None):
        """
        Sets the current session, user, and host files
        These are used as defaults by other methods.
        """
        if host is not None:
            self.host_file().write_text(host)
        if props is not None:
            port = props.get('omero.port', str(omero.constants.GLACIER2PORT))
            self.port_file().write_text(port)
        if name is not None:
            self.user_file(host).write_text(name)
            if uuid is not None:
                self.sess_file(host, name).write_text(uuid)

    def get_current(self):
        host = None
        name = None
        uuid = None
        if self.host_file().exists():
            host = self.host_file().text().strip()
        if host:
            try:
                name = self.user_file(host).text().strip()
            except IOError:
                pass
        if name:
            try:
                uuid = self.sess_file(host, name).text().strip()
            except IOError:
                pass
        return (host, name, uuid, self.last_port())

    def last_host(self):
        """
        Prints either the last saved host (see get_current())
        or "localhost"
        """
        f = self.host_file()
        if not f.exists():
            return "localhost"
        text = f.text().strip()
        if not text:
            return "localhost"
        return text

    def last_port(self):
        """
        Prints either the last saved port (see get_current())
        or "4064"
        """
        f = self.port_file()
        if not f.exists():
            return str(omero.constants.GLACIER2PORT)
        port = f.text().strip()
        if not port:
            return str(omero.constants.GLACIER2PORT)
        return port

    def find_name_by_key(self, server, uuid):
        """
        Returns the name of a user for which the
        session key exists. This value is taken
        from the path rather than from the properties
        file since that value may have been overwritten.
        An exception is raised if there is more than one
        name since keys should be UUIDs. A None may be
        returned.
        """
        s = old_div(self.dir, server)
        if not s.exists():
            return None
        else:
            n = [x.basename() for x in s.dirs() if (old_div(x, uuid)).exists()]
            if not n:
                return None
            elif len(n) == 1:
                return n[0]
            else:
                raise Exception(
                    "Multiple names found for uuid=%s: %s"
                    % (uuid, ", ".join(n)))

    def contents(self):
        """
        Returns a map of maps with all the contents
        of the store. Internal accounting files are
        skipped.
        """
        rv = {}
        Dhosts = self.dir.dirs()
        for Dhost in Dhosts:
            host = unquote(str(Dhost.basename()))
            if host not in rv:
                rv[host] = {}
            Dnames = Dhost.dirs()
            for Dname in Dnames:
                name = str(Dname.basename())
                if name not in rv[host]:
                    rv[host][name] = {}
                Dids = self.non_dot(Dname)
                for Did in Dids:
                    id = str(Did.basename())
                    props = self.props(Did)
                    props["active"] = "unknown"
                    rv[host][name][id] = props
        return rv

    def count(self, host=None, name=None):
        """
        Returns the sum of all files visited by walk()
        """
        def f(h, n, s):
            f.i += 1
        f.i = 0
        self.walk(f, host, name)
        return f.i

    def walk(self, func, host=None, name=None, sess=None):
        """
        Applies func to all host, name, and session path-objects.
        """
        for h in self.dir.dirs():
            if host is None or str(h.basename()) == host:
                for n in h.dirs():
                    if name is None or str(n.basename()) == name:
                        for s in self.non_dot(n):
                            if sess is None or str(s.basename()) == sess:
                                func(h, n, s)

    #
    # Server-requiring methods
    #

    def attach(self, server, name, sess, set_current=True):
        """
        Simple helper. Delegates to create() using the session
        as both the username and the password. This reproduces
        the logic of client.joinSession()
        """
        props = self.get(server, name, sess)
        return self.create(sess, sess, props, new=False,
                           set_current=set_current)

    def create(self, name, pasw, props, new=True, set_current=True, sudo=None):
        """
        Creates a new omero.client object, and returns:
        (cilent, session_id, timeToIdle, timeToLive)
        """
        import omero.clients
        props = dict(props)
        host = props["omero.host"]
        client = omero.client(props)
        client.setAgent("OMERO.sessions")

        try:
            if sudo is not None:
                sf = client.createSession(sudo, pasw)
                principal = omero.sys.Principal()
                principal.name = name
                principal.group = props.get("omero.group", None)
                principal.eventType = "User"

                # Retrieve the default time to idle value
                uuid = sf.ice_getIdentity().name
                sess = sf.getSessionService().getSession(uuid)
                timeToIdle = sess.getTimeToIdle().getValue()

                sess = sf.getSessionService().createSessionWithTimeouts(
                    principal, 0, timeToIdle)
                client.closeSession()
                sf = client.joinSession(sess.getUuid().getValue())
            else:
                sf = client.createSession(name, pasw)
        except:
            client.__del__()
            raise

        ec = sf.getAdminService().getEventContext()
        uuid = sf.ice_getIdentity().name
        sf.detachOnDestroy()
        sess = sf.getSessionService().getSession(uuid)
        timeToIdle = sess.getTimeToIdle().getValue()
        timeToLive = sess.getTimeToLive().getValue()

        # Retrieve timeout from properties
        timeout = None
        if props.get("omero.timeout", False):
            timeout = int(props.get("omero.timeout")) * 1000

        # Update timeout
        if timeout and timeout != timeToIdle:
            req = omero.cmd.UpdateSessionTimeoutRequest()
            req.session = sf.getAdminService().getEventContext().sessionUuid
            req.timeToIdle = rlong(timeout)

            try:
                cb = client.submit(req)  # Response is "OK"
                cb.close(True)
            except omero.CmdError as ce:
                self.logger.warn(str(ce.err))
            except:
                import traceback

                self.logger.error(traceback.format_exc())

            # Reload session
            sess = sf.getSessionService().getSession(uuid)
            timeToIdle = sess.getTimeToIdle().getValue()

        if new:
            self.add(host, ec.userName, uuid, props, sudo=sudo)
        if set_current:
            self.set_current(host, ec.userName, uuid, props)

        return client, uuid, timeToIdle, timeToLive

    def clear(self, host=None, name=None, sess=None):
        """
        Walks through all sessions and calls killSession.
        Regardless of exceptions, it will remove the session files
        from the store.
        """
        removed = []

        def f(h, n, s):
            hS = str(h.basename())
            nS = str(n.basename())
            sS = str(s.basename())
            try:
                client = self.attach(hS, nS, sS)
                client.killSession()
            except Exception as e:
                self.logger.debug("Exception on killSession: %s" % e)
            s.remove()
            removed.append(s)
        self.walk(f, host, name, sess)
        return removed

    ##
    # Helpers. Do not modify or rely on mutable state.
    ##

    def host_file(self):
        """ Returns the path-object which stores the last active host """
        return old_div(self.dir, "._LASTHOST_")

    def port_file(self):
        """ Returns the path-object which stores the last active port """
        return old_div(self.dir, "._LASTPORT_")

    def user_file(self, host):
        """ Returns the path-object which stores the last active user """
        d = old_div(self.dir, _escape_host(host))
        if not d.exists():
            d.makedirs()
        return old_div(d, "._LASTUSER_")

    def sess_file(self, host, user):
        """ Returns the path-object which stores the last active session """
        d = self.dir / _escape_host(host) / user
        if not d.exists():
            d.makedirs()
        return old_div(d, "._LASTSESS_")

    def non_dot(self, d):
        """
        Only returns the files (not directories)
        contained in d that don't start with a dot
        """
        return [f for f in d.files("*")
                if not str(f.basename()).startswith(".")]

    def props(self, f):
        """
        Parses the path-object into properties
        """
        txt = f.text()
        lines = txt.split("\n")
        props = {}
        for line in lines:
            if not line:
                continue
            parts = line.split("=", 1)
            if len(parts) == 1:
                parts.append("")
            props[parts[0]] = parts[1]
        return props

if __name__ == "__main__":
    SessionsStore().report()
